/* Copyright (C) 2006 P.L. Lucas
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */

#include "terminal.h"
#include "config.h"
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QTextCursor>
#include <QLineEdit>
#include <QIcon>
#include <QTextStream>
#include <QMap>
#include <QLabel>
#include <QFileInfo>
#include <QDir>
#include <QDateTime>
#include "projects/projects.h"


Terminal::Terminal(QWidget * parent):BaseWidget(parent)
{
	widget_type=TERMINAL;
	
	octave_connection=NULL;
	
	menuBar()->hide();
	
	//Se crea la ventana del terminal
	setWindowTitle("Octave Terminal");
	setWindowIcon( QIcon(QString( ICON_PATH )+"/konsole.png"));
	
	if(get_config("lines_in_terminal").isEmpty())
	{
		QMap<QString,QString> c;
		c["lines_in_terminal"]="1000";
		set_config(c);
	}
	if(get_config("cols_in_terminal").isEmpty())
	{
		QMap<QString,QString> c;
		c["cols_in_terminal"]="80";
		set_config(c);
	}
	
	lines_in_terminal=get_config("lines_in_terminal").toInt();
	cols_in_terminal=get_config("cols_in_terminal").toInt();
	text=new QTextEdit(this);
	text->setLineWrapMode (QTextEdit::NoWrap);
	text->setReadOnly (true);
	if(get_config("terminal_font").isEmpty())
	{
		QMap<QString,QString> c;
		c["terminal_font"]="Courier 10 Pitch";
		set_config(c);
	}
	
	{
		QFont font;
		font.fromString(get_config("terminal_font"));
		text->setFont( font );
	}
	if(get_config("terminal_foreground_color").isEmpty())
	{
		QMap<QString,QString> c;
		c["terminal_foreground_color"]="Black";
		set_config(c);
	}
	
	if(get_config("terminal_background_color").isEmpty())
	{
		QMap<QString,QString> c;
		c["terminal_background_color"]="White";
		set_config(c);
	}
	
	if(get_config("terminal_error_color").isEmpty())
	{
		QMap<QString,QString> c;
		c["terminal_error_color"]="Red";
		set_config(c);
	}
	
	text->setTextColor(get_config("terminal_foreground_color"));
	{
		QPalette p=text->viewport ()->palette();
		p.setColor(QPalette::Base, QColor(get_config("terminal_background_color")) );
		text->viewport()->setPalette(p);
	}
	text->setHtml("Starting Octave...<br>");
	text->show();
	
	combo_box=new Autocomplete(this);
	//combo_box->setEditable(true);
	//combo_box->setAutoCompletion(false);
	//combo_box->setDuplicatesEnabled(true);
	combo_box->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
	//combo_box->load_from_file(CONFIG_PATH "/autocomplete.lst");
	combo_box->show();
	
	//Se captura el "intro" para pasar comandos al octave
	QLineEdit *line_edit=combo_box/*->lineEdit()*/;
	connect(line_edit, SIGNAL(returnPressed()), this, SLOT(return_pressed()));
	connect( line_edit, SIGNAL( textChanged ( const QString &) ) , this, SLOT( textChanged ( const QString &) ) );
	
	line_edit->setText(tr("Insert your commands here..."));
	line_edit->selectAll();
	line_edit->setToolTip(tr("Insert your commands here. Use arrows and tab key to commands navigation."));
	
	connect(combo_box, SIGNAL(selectionChanged ()), this, SLOT(clear_command_line_first_time()));
	
	/*
	completion_matches_button=new QPushButton(this);
	completion_matches_button->setIcon(QIcon(QString(ICON_PATH)+"/images/search.png"));
	completion_matches_button->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
	completion_matches_button->setToolTip(
		tr("<b>Completion matches:</b><br> Generates possible completions given in comand line.")
		);
	completion_matches_button->show();
	connect(completion_matches_button, SIGNAL(clicked()), this, SLOT(completion_matches_callback()));
	
	
	dynamic_help_button=new QPushButton(this);
	dynamic_help_button->setIcon(QIcon(QString(ICON_PATH)+"/images/help_index.png"));
	dynamic_help_button->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
	dynamic_help_button->setToolTip(
			tr("<b>Dynamic Help:</b><br> Shows help of completions matches.")
					     );
	dynamic_help_button->show();
	connect(dynamic_help_button, SIGNAL(clicked()), this, SLOT(dynamic_help_callback()));
	
	stop_process_button=new QPushButton(this);
	stop_process_button->setIcon(QIcon(QString(ICON_PATH)+"/images/button_cancel.png"));
	stop_process_button->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
	stop_process_button->setToolTip(
			tr("<b>Stops process:</b><br>Stops uncontroled process, infinite loops,...")
				       );
	stop_process_button->show();
	connect(stop_process_button, SIGNAL(clicked()), this, SLOT(stop_process_callback()));
	
	clear_button=new QPushButton(this);
	clear_button->setIcon(QIcon(QString(ICON_PATH)+"/images/eraser.png"));
	clear_button->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
	clear_button->setToolTip(
			tr("<b>Clear terminal:</b><br>Erase terminal screen.")
				       );
	clear_button->show();
	connect(clear_button, SIGNAL(clicked()), this, SLOT(clear_callback()));
	*/
	
	QVBoxLayout *layout = new QVBoxLayout;
	//QHBoxLayout *comand_layout = new QHBoxLayout;
	QHBoxLayout *line_layout = new QHBoxLayout;
	layout->addWidget(text);
	layout->addLayout(line_layout);
	/*
	layout->addLayout(comand_layout);
	comand_layout->addWidget(completion_matches_button);
	comand_layout->addWidget(dynamic_help_button);
	comand_layout->addWidget(stop_process_button);
	comand_layout->addWidget(clear_button);
	comand_layout->addWidget(new QWidget);
	*/
	
	line_layout->addWidget(new QLabel("<b>&gt;&gt;</b>",this));
	line_layout->addWidget(combo_box);
	
	centralWidget()->setLayout(layout);
	
	setAcceptDrops(true);
}

void Terminal::dragEnterEvent(QDragEnterEvent *event)
{
	if (event->mimeData()->hasFormat("text/plain"))
		event->acceptProposedAction();
}


void Terminal::dropEvent ( QDropEvent * event )
{
	QString path=event->mimeData()->text();
	
	QFileInfo fileInfo(path);
	
	QString cmd;
	
	// Cambiar directorio
	cmd = QString("cd ") + fileInfo.dir().absolutePath();
	octave_connection->command_enter(cmd);
	
	// Ejecutar
	cmd = fileInfo.baseName();
	octave_connection->command_enter(cmd);

	event->acceptProposedAction();
}

void Terminal::return_pressed()
{
	QLineEdit *line_edit=combo_box/*->lineEdit()*/;
	QString command(line_edit->text());
	command_enter(command);
};

QTextEdit *Terminal::getTextEdit()
{
	return text;
}

Autocomplete *Terminal::getAutocomplete()
{
	return combo_box;
}

void Terminal::setOctaveConnection(OctaveConnection *octave_connection)
{
	this->octave_connection=octave_connection;
	//octave_connection->setTextEdit(text);
	QObject::connect(octave_connection, SIGNAL(error_ready(QString)), this, SLOT(write_error(QString)));
	QObject::connect(octave_connection, SIGNAL(output_ready(QString)), this, SLOT(write_output(QString)));
	QObject::connect(octave_connection, SIGNAL(command_ready(QString)), this, SLOT(write_command(QString)));
	combo_box->set_octave_connection(octave_connection);
	//setProject(); //This line loads variables
}

OctaveConnection *Terminal::getOctaveConnection()
{
	return octave_connection;
}

void Terminal::command_enter (const QString &command)
{
	octave_connection->command_enter(command);
	/*combo_box->clearEditText();*/
	combo_box->setText("");
}

#include <QTextBlock>

void Terminal::remove_lines(QTextCursor &cursor)
{
	//Se cortan las l�eas que sean demasiado largas
	cursor.movePosition(QTextCursor::Start);
	int lines=0;
	while(true)
	{
		QTextBlock block=cursor.block();
		if(block.length()>cols_in_terminal)
		{
			cursor.movePosition(QTextCursor::StartOfBlock);
			cursor.movePosition(QTextCursor::Right,QTextCursor::MoveAnchor,cols_in_terminal-1);
			cursor.insertText("\n");
		}
		else
		{
			cursor.movePosition(QTextCursor::Down);
		}
		cursor.movePosition(QTextCursor::EndOfBlock);
		lines++;
		if(cursor.atEnd())
		{
			QTextBlock block=cursor.block();
			if(block.length()<cols_in_terminal) break;
		}
	}
	
	//Se borran las l�eas iniciales si hay m� de lines_in_terminal l�eas
	//printf("lines=%d %d\n",lines,lines_in_terminal);
	lines=text->toPlainText().count('\n');
	while(lines>lines_in_terminal)
	{
		cursor.movePosition(QTextCursor::Start);
		cursor.movePosition(QTextCursor::EndOfBlock,QTextCursor::KeepAnchor);
		//printf("line %d: %s\n",lines, cursor.selectedText().toLocal8Bit().data());
		if(cursor.hasSelection()) cursor.removeSelectedText();
		else cursor.deleteChar();
		cursor.deleteChar();
		lines--;
	}
	//printf("lines=%d %d\n",lines,lines_in_terminal);
}

void Terminal::write_output(QString output)
{
	QTextCursor cursor=text->textCursor();
	cursor.beginEditBlock();
	cursor.movePosition(QTextCursor::End);
	text->setTextCursor( cursor );
	text->setTextColor( get_config("terminal_foreground_color") );
	text->setFontWeight ( QFont::Normal );
	text->insertPlainText(output);
	remove_lines(cursor);
	cursor.movePosition(QTextCursor::End);
	cursor.endEditBlock();
	text->setTextCursor( cursor );
}

void Terminal::write_error(QString error)
{
	QTextCursor cursor=text->textCursor();
	cursor.beginEditBlock();
	cursor.movePosition(QTextCursor::End);
	text->setTextCursor( cursor );
	text->setTextColor( get_config("terminal_error_color") );
	text->setFontWeight ( QFont::Normal );
	text->insertPlainText(error);
	remove_lines(cursor);
	cursor.movePosition(QTextCursor::End);
	cursor.endEditBlock();
	text->setTextCursor( cursor );
}

void Terminal::write_command(QString command)
{
	QTextCursor cursor=text->textCursor();
	cursor.beginEditBlock();
	cursor.movePosition(QTextCursor::End);
	text->setTextCursor( cursor );
	text->setTextColor( get_config("terminal_foreground_color") );
	text->setFontWeight ( QFont::Bold );
	text->insertPlainText(command);
	text->setFontWeight ( QFont::Normal );
	remove_lines(cursor);
	cursor.movePosition(QTextCursor::End);
	cursor.endEditBlock();
	text->setTextCursor( cursor );
}

void Terminal::closeEvent(QCloseEvent *event)
{
	if(session->getTools(TERMINAL).size()<2)
	{
		//Ignorar este evento
		event->ignore();
	}
}

void Terminal::completion_matches_callback()
{
	QString command;
	QTextStream(&command) << "completion_matches(\"" << combo_box/*->lineEdit()*/->text() << "\")";
	
	octave_connection->command_enter(command,false);
}


#include <sys/types.h>
#include <signal.h>

void Terminal::stop_process_callback()
{
	#if defined Q_OS_WIN32
		//system(QString("sh kill -s SIGINT "+QString("%d").arg(octave_connection->pid()) ).toLocal8Bit().data() );
		octave_connection->terminate();
	#else
		kill(octave_connection->pid(),SIGINT);
	#endif
}

void Terminal::clear_callback()
{
	text->clear();
	octave_connection->command_enter("",false);
}

void Terminal::clear_command_line_first_time()
{
	combo_box->setText("");
	disconnect(combo_box, SIGNAL(selectionChanged ()), this, SLOT(clear_command_line_first_time()));
}

CommandList * Terminal::show_commands_callback()
{
	CommandList *command_list=new CommandList(work_space);
	connect(combo_box, SIGNAL(new_command_entered(QStringList)), command_list, SLOT(set_list(QStringList)) );
	command_list->setSession(session);
	command_list->setLineEdit(combo_box);
	command_list->set_list(combo_box->commands());
	if(work_space!=NULL) work_space->addSubWindow(command_list);
	command_list->show();
	return command_list;
}

void Terminal::textChanged( const QString & text)
{
	emit dynamic_help_required(text);
}

void Terminal::setSession(Session *session)
{
	BaseWidget::setSession(session);
	connect(session, SIGNAL(projectChanged(QString)), this, SLOT(setProject(QString)));
}

void Terminal::setProject(QString /*project*/)
{
	if(octave_connection!=NULL)
	{
		//Load variables
		QString file_name=Projects::variablesListPath(session->getProjectName());
		if(file_name.isEmpty()) return;
		QString command("load(\"-text\",\""+file_name+"\"); cd(qtoctave_pwd); clear qtoctave_pwd;");
		octave_connection->command_enter(command);
	}
}

Terminal::~Terminal()
{
	//saveVariables();
}

void Terminal::saveVariables()
{
	if(octave_connection!=NULL)
	{
		//Save variables
		QDateTime date=QDateTime::currentDateTime();
		QString file_name=QString::number( date.toTime_t() );
		
		QString home=QDir::home().path()+"/.qtoctave/";
		file_name=home+file_name+".variables";
		
		QString command("qtoctave_pwd=pwd; save(\"-text\",\""+file_name+"\");");
		octave_connection->command_enter(command);
		octave_connection->command_enter("disp(\"\\n\\nAll data has been saved.\");",false);
		
		//Octave has 1 seconds to save data
		Sleep::micro_sleep(1000000);
		
		Projects::saveVariablesListPath(session->getProjectName(), file_name);
	}
}

BaseWidget *Terminal::copyBaseWidget(QWidget * parent )
{
	Terminal *bw=new Terminal(parent);
	bw->setSession(session);
	bw->setProject();
	bw->setOctaveConnection(octave_connection);
	bw->text->setHtml(text->toHtml());
	QTextCursor cursor=bw->text->textCursor();
	cursor.movePosition(QTextCursor::End);
	bw->text->setTextCursor( cursor );
	
	return bw;
}



